% Calculate acoustic field in wide tube (BL approx) for given p_in and U_in
% (flow rate) and frequency over a domain length L.
% For transfer matrices and explanation see Joris appendix C.2 page 118 of his thesis). This is based on the BL
% approximation as given in the book of Swift. With his equation C.1 and C.2 from Swift 4.54 and 4.70, respectively
% The acoustic dissipation then follows from 5.11 page 121 of Swift (new book).
function [p1x, u1x, E2x, dE2, x] = CalcAcousticField(p_in, U_in, f, L, settings)

T0 = settings.physics.T0;
A0 = pi*settings.geom.R0^2;
p0 = settings.physics.P0;

omega = 2*pi*f;
rho = rho_0_T(T0,p0);
c0 = c_0_T(T0);
mu = mu_T(T0);
kappa = kappa_T(T0);
Cp = c_p_T(T0);
gamma = 1.40;


delta_nu = sqrt(2*mu/(omega*rho));
delta_kappa = sqrt(2*kappa/(Cp*omega*rho));
f_nu = (1-1i)*delta_nu/(settings.geom.R0); 
f_kappa = (1-1i)*delta_kappa/(settings.geom.R0); 

k = omega/c0*sqrt((1+(gamma-1)*f_kappa)/(1-f_nu));

% x array
x = min(0,L):5e-4:max(0,L);

u_in = U_in/A0;

% transfer matrix (p,u)_in --> (p,u)_out
T{1,1} = cos(k*x);
T{1,2} = -sin(k*x)*(1i*omega*rho)/((1-f_nu)*k);
T{2,1} = -sin(k*x)*(1i*(1-f_nu)*k)/(omega*rho);
T{2,2} = cos(k*x);

p1x = T{1,1}*p_in + T{1,2}*u_in;
u1x = T{2,1}*p_in + T{2,2}*u_in;

%Equation 5.11 page 121 of Swift book (new book)
dE2dx = (-0.25*rho*abs(u1x).^2*delta_nu*omega - 0.25*abs(p1x).^2/(gamma*p0)*(gamma-1)*delta_kappa*omega)*2*pi*settings.geom.R0;
for i = 2:length(x)
    E2x(i) = trapz(x(1:i),dE2dx(1:i));
end

dE2 = E2x(1)-E2x(end);

end