function E2 = CalcAcousticPower(P1,P2,phi,dx,freq,settings,method,varargin)
% Calculate acoustic power based on dp/dx method
% Use method switch to select different equations:
% 1) Original equation used by Kees de Blok, no attenuation included
% 2) Equation with attenuation effects (Eq. 3 in Fusco1991)
% 3) Equation for arbitrary large dx and attenuation effects (Eq. 23 in Fusco19991)

if nargin>6 && sum(strcmp(varargin,'silent'))
    doSilent = 1;
else
    doSilent = 0;
end

T0 = settings.physics.T0;
P0 = settings.physics.P0;

% Input parameters:
rho0 = rho_0_T(T0,P0); % [kg/m^s] mean density (air)
c0 = c_0_T(T0); % [m/s] mean speed of sound (air)
mu0 = mu_T(T0); % viscosity (air)
gamma = 1.4; % ratio of specific heats (air)
sigma = Pr_T(T0); % Prandtl number

D = 60e-3; % [m] tube diameter

%% Start calculations:
phi = phi/360*(2*pi); % [rad] 
lambda = c0/freq; % [m] wavelength
omega = 2*pi*freq;
delta_nu = sqrt(2*mu0/(omega*rho0));

if dx >= 0.05*lambda && method~=3 % check sensor distance
    warning(['Sensor mutual distance too large for accurate application of this method, select method (3): dx/lambda=' num2str(dx/lambda)]);
else
    if ~doSilent
        disp(['Sensor distance checked, dx < 0.05*lambda: dx/lambda=' num2str(dx/lambda)]);
    end
end

switch method
    case 1
        E2 = (pi*(D/2)^2/(2*omega*rho0*dx))*abs(P1)*abs(P2)*sin(phi); % Eq.2 in Fusco1991
    case 2
        E2 = (pi*(D/2)^2/(2*omega*rho0*dx))*((1-delta_nu/(D/2))*abs(P1)*abs(P2)*sin(phi) + delta_nu/D*(abs(P1)^2-abs(P2)^2)); % Eq.3 in Fusco1991
    case 3
        E2 = (pi*(D/2)^2)/(2*rho0*c0*sin(omega*dx/c0))*(abs(P1)*abs(P2)*sin(phi)*(1-delta_nu/D*(1-(gamma-1)/sqrt(sigma)+(1+(gamma-1)/sqrt(sigma))*omega*dx/c0*cot(omega*dx/c0))) + delta_nu/(2*D)*(abs(P1)^2-abs(P2)^2)*(1-(gamma-1)/sqrt(sigma)+(1+(gamma-1)/sqrt(sigma))*omega*dx/c0*csc(omega*dx/c0))); % Eq. 23 in Fusco1991
    otherwise
        error('ERROR: invalid method for acoustic power calculation selected!');
end

%% Output:
if ~doSilent
    disp(['Input: dx=' num2str(dx) ' m']);
    disp(['Input: pa_1=' num2str(P1) ' Pa']);
    disp(['Input: pa_2=' num2str(P2) ' Pa']);
    disp(['Input: phi=' num2str(phi/pi*180) ' deg']);
    switch method
        case 1
            disp('Used equation: Eq. 2 from Fusco1991');
        case 2
            disp('Used equation: Eq. 3 from Fusco1991');
        case 3
            disp('Used equation: Eq. 23 from Fusco1991');
    end
    disp('------- OUTPUT -----------------------------');
    disp(['Calculated viscous penetration depth: delta_nu=' num2str(delta_nu) 'm']);
    disp(['Calculated acoustic power: E2=' num2str(E2) ' W']);
end