function U = CalcFlowRate(pA,pB,dx,f,settings,R0,method)

% Calculates acoustic flow rate (U = u*A) based on two complex pressure
% amplitudes pA & pB and mutual distance dx. Assumes air at T0. Different
% methods available:
% 'simple': uses u=(pB-pA)/(dx*i*omega*rho)
% 'fusco': includes dissipative effects and valid for arbitrary dx

T0 = settings.physics.T0;
P0 = settings.physics.P0;

omega = 2*pi*f;
rho = rho_0_T(T0,P0);
c0 = c_0_T(T0);
mu = mu_T(T0);
kappa = kappa_T(T0);
Cp = c_p_T(T0);
gamma = 1.40;

if isreal(pA) && isreal(pB)
    warning('Pure real pressure amplitudes specified, no phase lag?!');
end

switch method
    
    case 'simple'
        u = -(pB-pA)./(dx*1i*omega*rho);
        U = u.*(pi*R0^2);
        
    case 'fusco'
        dnu = sqrt(2*mu./(omega*rho)); % viscous penetration depth
        dkappa = sqrt(2*kappa./(omega*rho*Cp)); % thermal penetration depth
        
        k = omega./c0.*(1+(1-1i)/2.*dnu/R0+(1-1i)/2.*(gamma-1).*(dkappa/R0)); % wave vector. Eq. 22, Fusco(1991)
        dpdx = k.*(pB-pA)./(2*sin(k*dx/2)); % pressure gradient, Eq. 20 Fusco(1991)
        U = (pi*dpdx.*(dnu.^2.*exp((R0*(- 1 - 1i))./dnu)*1i + dnu*R0*(- 1 + 1i) - dnu.^2*1i + R0^2)*1i)./(rho*omega); %Integrate Eq.12 Fusco(1991) over area (remember extra r in polar: U = int(int(u*r*dr*dtheta))) 
        %Alternatively, find terms of U from Eq.13 (Fusco(1991) as: U = (pi*R0^2/(rho*omega))*1i*dpdx*(1-((1-1i)*dnu)/R0); in practice, this gives the same answer
        
    otherwise
        error('Invalid method specified');
end

end