% Calculate pressure field in wide tube (BL approx) for given p_in and
% p_out and frequency over a domain length L.
% For transfer matrices and explanation see Joris appendix C.2 page 118 of his thesis). This is based on the BL
% approximation as given in the book of Swift. With his equation C.1 and C.2 from Swift 4.54 and 4.70, respectively
function [p1x, x] = CalcPressureField(p_in, p_out, f, L, settings)

T0 = settings.physics.T0;
P0 = settings.physics.P0;

omega = 2*pi*f;
rho = rho_0_T(T0,P0);
c0 = c_0_T(T0);
mu = mu_T(T0);
kappa = kappa_T(T0);
Cp = c_p_T(T0);
gamma = 1.40;

delta_nu = sqrt(2*mu/(omega*rho));
delta_kappa = sqrt(2*kappa/(Cp*omega*rho));
f_nu = (1-1i)*delta_nu/(settings.geom.R0);   
f_kappa = (1-1i)*delta_kappa/(settings.geom.R0); 

k = omega/c0*sqrt((1+(gamma-1)*f_kappa)/(1-f_nu));

% x array
x = min(0,L):5e-4:max(0,L);
%Fix PressureLocation strange error that sometimes last value in x-array (above) is not present.
if sum(x==L)==0  %The last element x==L is somehow not made, so add it manually
    x = [x L];
end

% transfer matrix (p,u)_in --> (p,u)_out
T{1,1} = cos(k*x);
T{1,2} = -sin(k*x)*(1i*omega*rho)/((1-f_nu)*k);
T{2,1} = -sin(k*x)*(1i*(1-f_nu)*k)/(omega*rho);
T{2,2} = cos(k*x);

p1x = T{1,1}*p_in + T{1,2}*(p_out-T{1,1}(x==L)*p_in)/T{1,2}(x==L);

end