% Processes single experimental TDMS data from LabVIEW
% All script are produced and work on MATLAB R2014a
% Run the script to select and process experimental data

%% Init

if isempty(whos('file_n')) %Serial/single processing (if file_n variable doesn't exist)
    if ~isempty(whos('data'))
        settings.data.doLoadData = questdlg('Do you want to re-initialize and select a new data file?','Initialization', 'No');
    else
        disp('No data currently loaded, initialize and select data file...');
        settings.data.doLoadData = 'Yes';
    end

    switch settings.data.doLoadData
        case 'Yes'
    %         clear all;
            clearvars -except RPM_eff_fits
    %         close all;
            settings.data.doLoadData = 'Yes'; %redefine toggle after clear all command
        case 'No'
            disp('Continue with previously loaded data...');
        otherwise
            error('Invalid doLoadData switch!');
    end
    settings.data.batch_processing = 0;
else %Batch processing (bi-pass dialog boxes)
    settings.data.batch_processing = 1;
end

%% Settings
%Add TDMS reader and required functions to path (must be in the same folder)
mydir = pwd; TDMS_path = [mydir '\TDMS reader\']; Functions_path = [mydir '\Functions\'];
addpath(TDMS_path); addpath(Functions_path);
settings.data.default_path = mydir;

settings.data.shiftTimestamp = 7/24; % number of days to add to LabVIEW timestamp to get current time
settings.data.PressureLocations = [500 700 950 1100]*10^-3; % locations of pressure sensors, x=0 at endpiece (identical to labview), number P1 to P4.  Since 181107 automatic from labview. From before, these values are used.  

settings.data.DiffPressureIndices = [3 2]; % indices of pressure sensors used for pressure drop calculation (first - second)
settings.data.Range =  inf; % select number of data points to load from end, use inf to load everything
settings.data.MaxPressure = 7.0e3; % maximum pressure to avoid clipping of sensors (p1 + p2). 180504: Sensor calibration stable up to 7000Pa. 
settings.data.PhaseMod = 180; % mod phase difference

%Phase shift between the sensors
settings.data.variable_phaseshift = 1; %Toggle for setting frequency specific phaseshift. If untoggled (==0) 0 phaseshift is set for all sensors. Note that this only works if only a single frequency is present per TDMS results file

settings.data.check_gen_output = 1; %Toggle for checking 1) the generator AC power output vs. RPM compared with the generator calibration and 2) if RPM is out of calibration range
settings.data.check_gen_output_ACtolerance = 4; %[default: 4] Maximum allowed percentage of deviation for AC electric power from calibration data.
settings.data.check_gen_output_max_delta = 1; %[default: 1] Maximum allowed percentage of larger power deviation compared to average (only invalid point if larger). Used to filter single points that are higher outliers, but within the set ACtolerance
settings.calc.gen_power_method = 'fourier'; %Choose: 'fourier' or 'direct'. How RMS voltage is calculated. Either amplitude from fft or a direct RMS of the data. 

settings.calc.allowedTimeDev = 2; % allowed relative deviation in timesteps (to prevent merging multiple data series occasionally)
settings.calc.doTimeIntervalCheck = 1; % toggle check of time interval

settings.calc.tol = 1; % accuracy of operation point detection, adapt to chosen variable!
settings.calc.SeparationVar = 'VolFreq'; % set variable to separate operation points: 'P4' or 'VolFreq'
settings.calc.PhaseTol = 360; % [deg] bounds for outlier detection in phase information
settings.calc.MinBinSize = 10; % minimum bin size for operation point detection
settings.calc.PreBufferSize = 10; % number of data points to skip before operation point shift
settings.calc.PostBufferSize = 2; % number of data points to skip after operation point shift

settings.calc.AverageMethod = 'mean'; % use 'mean' or 'median'
settings.calc.E2AverageMethod = 'pointwise'; % use 'average' or 'pointwise'
settings.calc.E2CalcMethod = 3; % use 1) for basic equation, 2) for attenuation, 3) for arbitrary large dx

settings.geom.R0 = 30e-3; % [m] radius outer tube

%Turbine specifications
settings.geom.Lrotor = 10.60*10^-3; %[m] Axial length of a the rotor;
settings.geom.Lguidevane = 10.0*10^-3; %[m] Axial length of a single stator/guidevane;
settings.geom.Lcone = 40*10^-3; %[m] Axial length of a single turbine cone;
settings.geom.Lgen_block = 0.8*10^-3; %[m] Axial length of additional length of left guidevane to place generator on;
settings.geom.Lspacing = 1*10^-3; %[m] Axial length of spacing between rotor and stator;
settings.geom.Lturbine = 2*settings.geom.Lcone+2*settings.geom.Lguidevane+2*settings.geom.Lspacing+settings.geom.Lrotor+settings.geom.Lgen_block; % [m] Turbine length including spacing
settings.data.xleft_guidevane = 0.850; % [m] The position of the most leftward point of the left guidevane blades

settings.data.xturbine_start = settings.data.xleft_guidevane-settings.geom.Lgen_block-settings.geom.Lcone; % [m] Axial turbine location for acoustic power correction and pressure drop over turbine; most leftward point. Add settings.geom.Lturbine for most rightward point 

%% Load data

if isempty(whos('file_n')) %Serial/single processing (if file_n variable doesn't exist)
    switch settings.data.doLoadData
        case 'Yes'
            [settings.data.filename, settings.data.pathname] = uigetfile('*.tdms','Select TDMS results file',settings.data.default_path);
            disp(['Loading data file ' settings.data.filename ' ...']);
            data = TDMS_getStruct([settings.data.pathname '\' settings.data.filename]); % load TDMS file
        case 'No'
            disp(['Keep current data file: ' settings.data.filename])
        otherwise
            error('Invalid doLoadData switch!');
    end
    clearvars -except settings data RPM_eff_fits;
else %Batch processing (bi-pass dialog boxes)
    disp(['Loading data file ' current_filename ' in batch procesing mode...']);
    data = TDMS_getStruct([settings.data.Batch.pathname current_filename]); % load TDMS file
end

%% Pre-processing

% Get channel information
settings.data.channels = fieldnames(data.DC_value); % get channel names from data
settings.data.channels = settings.data.channels(3:end); % remove 'name' and 'Props' fields
settings.data.Nch = length(settings.data.channels); % get number of channels
if strcmp(settings.data.channels(1),'Voltage') && strcmp(settings.data.channels(2),'Current') % check for correct channel naming
    if settings.data.batch_processing == 0 %Don't show in batch processing mode
        disp('First two channels successfully recognized as Voltage & Current')
    end
else
    error('ERROR: power channels not correctly recognized, check structure of data file');
end

% Process time data
absTime = data.Time.Time.data(max(end-settings.data.Range,1):end)+settings.data.shiftTimestamp;
time = (absTime-absTime(1))*86400; % relative time in seconds
settings.data.Nt = length(time);
settings.data.dtAvg = (time(end)-time(1))/settings.data.Nt;
if settings.calc.doTimeIntervalCheck
    for i = 2:settings.data.Nt-1 % check for (almost) uniform time interval. % Skip first time point since there might be a delay to get the turbine running before starting the actual experiment
        if time(i+1)-time(i) > settings.calc.allowedTimeDev*(time(end)-time(end-1)) % || time(i+1)-time(i) < 1/settings.calc.allowedTimeDev*(time(end)-time(end-1))
            error(['ERROR: too large deviation in time interval found between ' datestr(absTime(i)) ' and ' datestr(absTime(i+1))]);
        end
    end
else
    warning('Check for constant time interval is disabled! Use settings.calc.doTimeIntervalCheck to toggle.');
end

% Get number of variables (groups)
settings.data.varnames = fieldnames(data);
settings.data.Nvar = length(settings.data.varnames);

%Get generator resistance
if isfield(data.Props,'Generator_resistance__Ohm_')
    settings.data.GeneratorResistance = data.Props.Generator_resistance__Ohm_;
else
    error('No generator resistance found in stored data structure. Processing aborted!')
end

%Get ambient conditions and calculate physical properties
settings.physics.T0 = data.Props.Ambient_temperature__C_ + 273.15; % [K] mean temperature 
settings.physics.P0 = data.Props.Ambient_pressure__hPa_*100; % [Pa] ambient pressure at start of measurement

%Get what generator and turbine set-up is used.
settings.geom.used_generator = ['gen' num2str(str2double(data.Props.Generator)+1)]; %Automatically get generator info (stored value+1 is actual generator)
settings.geom.guidevane_left = data.Props.Guidevane_left_;
settings.geom.guidevane_right = data.Props.Guidevane_right_;
settings.geom.rotor = data.Props.Rotor;

%Get the pressure sensor location (available from 181107 onwards; otherwise use the one in settings).
if isfield(data.Props,'Pressure_sensor_location_1__mm_')
    eval(['settings.data.PressureLocations = [' num2str(data.Props.Pressure_sensor_location_1__mm_) ' ' num2str(data.Props.Pressure_sensor_location_2__mm_) ' ' num2str(data.Props.Pressure_sensor_location_3__mm_) ' ' num2str(data.Props.Pressure_sensor_location_4__mm_) ']*10^-3;' ])
else
    disp('No pressure sensor location found from Labview. Using the values specified in the settings section') 
end

%Set physical parameters according to T0, P0
settings.physics.rho0 = rho_0_T(settings.physics.T0,settings.physics.P0);
settings.physics.mu = mu_T(settings.physics.T0);
settings.physics.nu = settings.physics.mu/settings.physics.rho0;
settings.physics.c0 = c_0_T(settings.physics.T0);
settings.physics.gamma = 1.4;
settings.physics.Pr = Pr_T(settings.physics.T0);

%Set sensor phaseshift
if settings.data.variable_phaseshift == 1
    if round(median(data.frequency.Voltage.data)) == 40
        settings.data.PhaseShift = [0.06175	0.1021	0.07836	0.06434];
    elseif round(median(data.frequency.Voltage.data)) == 50
        settings.data.PhaseShift = [0.07364	0.1118	0.09519	0.08001];
    elseif round(median(data.frequency.Voltage.data)) == 60
        settings.data.PhaseShift = [0.08273	0.1313	0.1105	0.0902];
    elseif round(median(data.frequency.Voltage.data)) == 70
        settings.data.PhaseShift = [0.08631	0.139	0.1172	0.09354];
    elseif round(median(data.frequency.Voltage.data)) == 80
        settings.data.PhaseShift = [0.08241	0.1393	0.1181	0.0898];
    elseif round(median(data.frequency.Voltage.data)) == 90
        settings.data.PhaseShift = [0.06885	0.1241	0.1016	0.08937];
    elseif round(median(data.frequency.Voltage.data)) == 100
        settings.data.PhaseShift = [0.07279	0.1477	0.1207	0.08203];
    else
       warning('No phase calibration data present for this acoustic frequency. The phaseshift is set to 0 for all sensors') 
       settings.data.PhaseShift = [0 0 0 0]; %[deg] No phase difference correction.
    end
else
    settings.data.PhaseShift = [0 0 0 0]; %[deg] No phase difference correction.
end

%% Find constant operating points (based on variable specified in settings.calc.SeparationVar)
results.R.data = data.R_.Untitled.data(max(end-settings.data.Range,1):end);
results.p1.S1.data = data.amplitude.P1.data(max(end-settings.data.Range,1):end);
results.p1.S2.data = data.amplitude.P2.data(max(end-settings.data.Range,1):end);
results.p1.S3.data = data.amplitude.P3.data(max(end-settings.data.Range,1):end);
results.p1.S4.data = data.amplitude.P4.data(max(end-settings.data.Range,1):end);
results.p2.S1.data = data.DC_value.P1.data(max(end-settings.data.Range,1):end);
results.p2.S2.data = data.DC_value.P2.data(max(end-settings.data.Range,1):end);
results.p2.S3.data = data.DC_value.P3.data(max(end-settings.data.Range,1):end);
results.p2.S4.data = data.DC_value.P4.data(max(end-settings.data.Range,1):end);
results.V.data = data.amplitude.Voltage.data(max(end-settings.data.Range,1):end);
results.I.data = data.amplitude.Current.data(max(end-settings.data.Range,1):end);
results.Freq.data = data.frequency.Voltage.data(max(end-settings.data.Range,1):end);
results.Freq.input = data.Input.Frequency.data(max(end-settings.data.Range,1):end);
results.Vol.input =  data.Input.Volume.data(max(end-settings.data.Range,1):end);
results.Status.input = data.Input.Status.data(max(end-settings.data.Range,1):end);
results.E2.S1S2.data = data.E2___A.Untitled.data(max(end-settings.data.Range,1):end);
results.E2.S3S4.data = data.E2___B.Untitled.data(max(end-settings.data.Range,1):end);
results.phi.S1.data = data.phase.P1.data(max(end-settings.data.Range,1):end)-settings.data.PhaseShift(1);
results.phi.S2.data = data.phase.P2.data(max(end-settings.data.Range,1):end)-settings.data.PhaseShift(2);
results.phi.S3.data = data.phase.P3.data(max(end-settings.data.Range,1):end)-settings.data.PhaseShift(3);
results.phi.S4.data = data.phase.P4.data(max(end-settings.data.Range,1):end)-settings.data.PhaseShift(4);
results.phi.S1S2.data = results.phi.S2.data-results.phi.S1.data;
results.phi.S3S4.data = results.phi.S4.data-results.phi.S3.data;

switch settings.calc.gen_power_method
    case 'fourier'
        results.generator.RMSamp.data = data.Pico_Channel_A.RMS_amp.data(max(end-settings.data.Range,1):end);
    case 'direct'
        results.generator.RMSamp.data = data.Pico_Channel_A.RMS_amp_direct.data(max(end-settings.data.Range,1):end);
    otherwise
        warning('Invalid settings.calc.gen_power_method!');
end
results.generator.frequency.data = data.Pico_Channel_A.Frequency.data(max(end-settings.data.Range,1):end);

% Remove data points where pressure signal exceeds limit:
% first find indices to remove
ToRemove = zeros(size(results.p1.S1.data));
for i = 1:4
    ToRemove = max(ToRemove,results.p1.(['S' num2str(i)]).data+results.p2.(['S' num2str(i)]).data>settings.data.MaxPressure);
end
if sum(ToRemove)>0 % at least one point to remove
%     beep;
    warning(['Some data points above clipping limit of sensors found, ' num2str(sum(ToRemove)) ' points will be removed']);
%     pause;
end

% then remove data points
results.R.data = results.R.data(~ToRemove);
results.p1.S1.data = results.p1.S1.data(~ToRemove);
results.p1.S2.data = results.p1.S2.data(~ToRemove);
results.p1.S3.data = results.p1.S3.data(~ToRemove);
results.p1.S4.data = results.p1.S4.data(~ToRemove);
results.p2.S1.data = results.p2.S1.data(~ToRemove);
results.p2.S2.data = results.p2.S2.data(~ToRemove);
results.p2.S3.data = results.p2.S3.data(~ToRemove);
results.p2.S4.data = results.p2.S4.data(~ToRemove);
results.V.data = results.V.data(~ToRemove);
results.I.data = results.I.data(~ToRemove);
results.Freq.data = results.Freq.data(~ToRemove);
results.Freq.input = results.Freq.input(~ToRemove);
results.Vol.input =  results.Vol.input(~ToRemove);
results.Status.input = results.Status.input(~ToRemove);
results.E2.S1S2.data = results.E2.S1S2.data(~ToRemove);
results.E2.S3S4.data = results.E2.S3S4.data(~ToRemove);
results.phi.S1S2.data = results.phi.S1S2.data(~ToRemove);
results.phi.S3S4.data = results.phi.S3S4.data(~ToRemove);
results.phi.S1.data = results.phi.S1.data(~ToRemove);
results.phi.S2.data = results.phi.S2.data(~ToRemove);
results.phi.S3.data = results.phi.S3.data(~ToRemove);
results.phi.S4.data = results.phi.S4.data(~ToRemove);
time = time(~ToRemove);

results.generator.RMSamp.data = results.generator.RMSamp.data(~ToRemove);
results.generator.frequency.data = results.generator.frequency.data(~ToRemove);

% Filter outliers in phase information:
results.phi.S1S2.outliers = excludedata(results.Freq.data,results.phi.S1S2.data,'range',[-settings.calc.PhaseTol settings.calc.PhaseTol]);
results.phi.S1S2.data(results.phi.S1S2.outliers) = NaN;
results.phi.S3S4.outliers = excludedata(results.Freq.data,results.phi.S3S4.data,'range',[-settings.calc.PhaseTol settings.calc.PhaseTol]);
results.phi.S3S4.data(results.phi.S3S4.outliers) = NaN;
results.phi.S1.data(results.phi.S1S2.outliers) = NaN;
results.phi.S2.data(results.phi.S1S2.outliers) = NaN;
results.phi.S3.data(results.phi.S3S4.outliers) = NaN;
results.phi.S4.data(results.phi.S3S4.outliers) = NaN;

% Unwrap phase:
results.phi.S1.data = unwrap(results.phi.S1.data*2*pi/360,settings.data.PhaseMod/360*2*pi)/2/pi*360;
results.phi.S2.data = unwrap(results.phi.S2.data*2*pi/360,settings.data.PhaseMod/360*2*pi)/2/pi*360;
results.phi.S3.data = unwrap(results.phi.S3.data*2*pi/360,settings.data.PhaseMod/360*2*pi)/2/pi*360;
results.phi.S4.data = unwrap(results.phi.S4.data*2*pi/360,settings.data.PhaseMod/360*2*pi)/2/pi*360;
results.phi.S1S2.data = mod(results.phi.S1S2.data,settings.data.PhaseMod);
results.phi.S3S4.data = mod(results.phi.S3S4.data,settings.data.PhaseMod);

% Get pressure drop and acoustic power dissipation (difference variables):
results.p2.drop.data = results.p2.(['S' num2str(settings.data.DiffPressureIndices(1))]).data - results.p2.(['S' num2str(settings.data.DiffPressureIndices(2))]).data;
results.E2.drop.data = results.E2.S3S4.data-results.E2.S1S2.data;

% Convert status to numerical value:
% 0: 'Idle'
% 1: 'Creating waveforms'
% 2: 'Start playing'
% 3: 'Playing'
% 4: 'Stop playing'

for i = 1:length(time)
    switch results.Status.input{i}
        case 'Idle'
            results.Status.data(i) = 0;
        case 'Creating waveforms'
            results.Status.data(i) = 1;
        case 'Start playing'
            results.Status.data(i) = 2;
        case 'Playing'
            results.Status.data(i) = 3;
        case 'Stop playing'
            results.Status.data(i) = 4;
        otherwise
            results.Status.data(i) = NaN;
            warning('Invalid status detected in results.Status.input!');
    end
end


% Find constant operation points
k = 1; % bin counter
j = 1; % bin length counter
indices = [];
for i = 3:length(time)-1
    switch settings.calc.SeparationVar
        case 'P4'
            IsSameBin = abs(results.p1.S4.data(i)-results.p1.S4.data(i+1))<settings.calc.tol;
        case 'VolFreq'
            IsSameBin = abs(results.Vol.input(i)-results.Vol.input(i+1))<settings.calc.tol && abs(results.Freq.input(i)-results.Freq.input(i+1))<settings.calc.tol;
        otherwise
            error('ERROR: no valid variable selected to separate operation points, set settings.calc.SeparationVar!');
    end
    if  IsSameBin % next point belongs to same operating point
        indices(k,j) = i;
        j = j+1;
    else % move to next
        if j <= settings.calc.MinBinSize-1 && j>1 % detected bin too small, delete
            disp(['Bin with indices ' num2str(indices(k,1)) ':' num2str(indices(k,end)) ' too short, deleted']);
            indices(k,:) = 0;
        else
            % crop indices, remove buffer
            indices(k,1:settings.calc.PreBufferSize) = 0;
            indices(k,end-settings.calc.PostBufferSize:end) = 0;
            k = k+1;
        end
        j = 1;
    end
end
clear IsSameBin;
settings.data.Nsetpoint = size(indices,1);

% And don't forget to crop last setpoint...:
indices(settings.data.Nsetpoint,1:settings.calc.PreBufferSize) = 0;
indices(settings.data.Nsetpoint,end-settings.calc.PostBufferSize:end) = 0;

%% Recalculate acoustic power if pointwise method is selected:
if strcmp(settings.calc.E2AverageMethod,'pointwise')
    if settings.data.batch_processing == 0 %Don't show in batch processing mode
        disp('Recalculating acoustic power pointwise...');
    end
    for i = 1:length(time)
        results.E2.S1S2.recalcdata(i) = CalcAcousticPower(results.p1.S2.data(i),results.p1.S1.data(i),results.phi.S1S2.data(i),settings.data.PressureLocations(2)-settings.data.PressureLocations(1),results.Freq.data(i),settings,settings.calc.E2CalcMethod,'silent');
        results.E2.S3S4.recalcdata(i) = CalcAcousticPower(results.p1.S4.data(i),results.p1.S3.data(i),results.phi.S3S4.data(i),settings.data.PressureLocations(4)-settings.data.PressureLocations(3),results.Freq.data(i),settings,settings.calc.E2CalcMethod,'silent');
    end
end

%% Calculate mean & std. per operating point
i=1;
while i <= settings.data.Nsetpoint
    CurrIndices = indices(i,find(indices(i,:),1,'first'):find(indices(i,:),1,'last')); % find nonzero entries
    if ~isempty(CurrIndices)
        results.Freq.mean(i) = mean(results.Freq.data(CurrIndices));
        results.Vol.mean(i) = mean(results.Vol.input(CurrIndices));
        if std(results.Freq.data(CurrIndices)) > settings.calc.tol % check for deviation in frequency, not allowed
            warning('deviation in detected frequency found within one operating point!');
        end
        if std(results.Vol.input(CurrIndices)) > 0 % check for deviation in audio volume, not allowed
            warning('deviation in audio volume found within one operating point!');
        end
        if std(results.Freq.input(CurrIndices)) > 0% check for deviation in input frequency, not allowed
            warning('deviation in input frequency found within one operating point!');
        end
        switch settings.calc.AverageMethod
            case 'mean'
                results.R.mean(i) = mean(results.R.data(CurrIndices));
                results.p1.S1.mean(i) = mean(results.p1.S1.data(CurrIndices));
                results.p1.S2.mean(i) = mean(results.p1.S2.data(CurrIndices));
                results.p1.S3.mean(i) = mean(results.p1.S3.data(CurrIndices));
                results.p1.S4.mean(i) = mean(results.p1.S4.data(CurrIndices));
                results.p2.S1.mean(i) = mean(results.p2.S1.data(CurrIndices));
                results.p2.S2.mean(i) = mean(results.p2.S2.data(CurrIndices));
                results.p2.S3.mean(i) = mean(results.p2.S3.data(CurrIndices));
                results.p2.S4.mean(i) = mean(results.p2.S4.data(CurrIndices));
                results.p2.drop.mean(i) = mean(results.p2.drop.data(CurrIndices));
                results.V.mean(i) = mean(results.V.data(CurrIndices));
                results.I.mean(i) = mean(results.I.data(CurrIndices));
                results.E2.S1S2.mean(i) = mean(results.E2.S1S2.data(CurrIndices));
                results.E2.S3S4.mean(i) = mean(results.E2.S3S4.data(CurrIndices));
                results.E2.drop.mean(i) = mean(results.E2.drop.data(CurrIndices));
                results.phi.S1S2.mean(i) = nanmean(results.phi.S1S2.data(CurrIndices));
                results.phi.S3S4.mean(i) = nanmean(results.phi.S3S4.data(CurrIndices));
                results.phi.S1.mean(i) = nanmean(results.phi.S1.data(CurrIndices));
                results.phi.S2.mean(i) = nanmean(results.phi.S2.data(CurrIndices));
                results.phi.S3.mean(i) = nanmean(results.phi.S3.data(CurrIndices));
                results.phi.S4.mean(i) = nanmean(results.phi.S4.data(CurrIndices));
                
                results.generator.RMSamp.mean(i) = nanmean(results.generator.RMSamp.data(CurrIndices));
                results.generator.frequency.mean(i) = nanmean(results.generator.frequency.data(CurrIndices));
                
                if strcmp(settings.calc.E2AverageMethod,'pointwise')
                    results.E2.S1S2.calc(i) = mean(results.E2.S1S2.recalcdata(CurrIndices));
                    results.E2.S3S4.calc(i) = mean(results.E2.S3S4.recalcdata(CurrIndices));
                end
            case 'median'
                results.R.mean(i) = median(results.R.data(CurrIndices));
                results.p1.S1.mean(i) = median(results.p1.S1.data(CurrIndices));
                results.p1.S2.mean(i) = median(results.p1.S2.data(CurrIndices));
                results.p1.S3.mean(i) = median(results.p1.S3.data(CurrIndices));
                results.p1.S4.mean(i) = median(results.p1.S4.data(CurrIndices));
                results.p2.S1.mean(i) = median(results.p2.S1.data(CurrIndices));
                results.p2.S2.mean(i) = median(results.p2.S2.data(CurrIndices));
                results.p2.S3.mean(i) = median(results.p2.S3.data(CurrIndices));
                results.p2.S4.mean(i) = median(results.p2.S4.data(CurrIndices));
                results.p2.drop.mean(i) = median(results.p2.drop.data(CurrIndices));
                results.V.mean(i) = median(results.V.data(CurrIndices));
                results.I.mean(i) = median(results.I.data(CurrIndices));
                results.E2.S1S2.mean(i) = median(results.E2.S1S2.data(CurrIndices));
                results.E2.S3S4.mean(i) = median(results.E2.S3S4.data(CurrIndices));
                results.E2.drop.mean(i) = median(results.E2.drop.data(CurrIndices));
                results.phi.S1S2.mean(i) = nanmedian(results.phi.S1S2.data(CurrIndices));
                results.phi.S3S4.mean(i) = nanmedian(results.phi.S3S4.data(CurrIndices));
                results.phi.S1.mean(i) = nanmedian(results.phi.S1.data(CurrIndices));
                results.phi.S2.mean(i) = nanmedian(results.phi.S2.data(CurrIndices));
                results.phi.S3.mean(i) = nanmedian(results.phi.S3.data(CurrIndices));
                results.phi.S4.mean(i) = nanmedian(results.phi.S4.data(CurrIndices));
                
                results.generator.RMSamp.mean(i) = nanmedian(results.generator.RMSamp.data(CurrIndices));
                results.generator.frequency.mean(i) = nanmedian(results.generator.frequency.data(CurrIndices));
                
                if strcmp(settings.calc.E2AverageMethod,'pointwise')
                    results.E2.S1S2.calc(i) = median(results.E2.S1S2.recalcdata(CurrIndices));
                    results.E2.S3S4.calc(i) = median(results.E2.S3S4.recalcdata(CurrIndices));
                end
            otherwise
                error('ERROR: invalid averaging method specified in settings.calc.AverageMethod');
        end
        results.R.std(i) = std(results.R.data(CurrIndices));
        results.p1.S1.std(i) = std(results.p1.S1.data(CurrIndices));
        results.p1.S2.std(i) = std(results.p1.S2.data(CurrIndices));
        results.p1.S3.std(i) = std(results.p1.S3.data(CurrIndices));
        results.p1.S4.std(i) = std(results.p1.S4.data(CurrIndices));
        results.p2.S1.std(i) = std(results.p2.S1.data(CurrIndices));
        results.p2.S2.std(i) = std(results.p2.S2.data(CurrIndices));
        results.p2.S3.std(i) = std(results.p2.S3.data(CurrIndices));
        results.p2.S4.std(i) = std(results.p2.S4.data(CurrIndices));
        results.p2.drop.std(i) = std(results.p2.drop.data(CurrIndices));
        results.E2.drop.std(i) = std(results.E2.drop.data(CurrIndices));
        results.E2.S1S2.std(i) = std(results.E2.S1S2.data(CurrIndices));
        results.E2.S3S4.std(i) = std(results.E2.S3S4.data(CurrIndices));
        results.V.std(i) = std(results.V.data(CurrIndices));
        results.I.std(i) = std(results.I.data(CurrIndices));
        results.phi.S1S2.std(i) = nanstd(results.phi.S1S2.data(CurrIndices));
        results.phi.S1.std(i) = nanstd(results.phi.S1.data(CurrIndices));
        results.phi.S2.std(i) = nanstd(results.phi.S2.data(CurrIndices));
        results.phi.S3.std(i) = nanstd(results.phi.S3.data(CurrIndices));
        results.phi.S4.std(i) = nanstd(results.phi.S4.data(CurrIndices));
        results.phi.S3S4.std(i) = nanstd(results.phi.S3S4.data(CurrIndices));
        
        results.generator.RMSamp.std(i) = nanstd(results.generator.RMSamp.data(CurrIndices));
        results.generator.frequency.std(i) = nanstd(results.generator.frequency.data(CurrIndices));
                   
    i = i+1;
    else % empty CurrIndices, only zeros in indices, skip this operating point
        indices = indices([1:i-1 i+1:end],:);
        settings.data.Nsetpoint = size(indices,1);
        disp('WARNING: Empty operating point detected, deleted');
    end
    
end

% Recalculate acoustic power if average method is selected:
if strcmp(settings.calc.E2AverageMethod,'average')
    if settings.data.batch_processing == 0 %Don't show in batch processing mode
        disp('Recalculating acoustic power averaged...');
    end
    for i = 1:settings.data.Nsetpoint
        results.E2.S1S2.calc(i) = CalcAcousticPower(results.p1.S2.mean(i),results.p1.S1.mean(i),results.phi.S1S2.mean(i),settings.data.PressureLocations(2)-settings.data.PressureLocations(1),results.Freq.mean(i),settings,settings.calc.E2CalcMethod,'silent');
        results.E2.S3S4.calc(i) = CalcAcousticPower(results.p1.S4.mean(i),results.p1.S3.mean(i),results.phi.S3S4.mean(i),settings.data.PressureLocations(4)-settings.data.PressureLocations(3),results.Freq.mean(i),settings,settings.calc.E2CalcMethod,'silent');
    end
end

%% Calculate acoustic power dissipation, complex pressure drop and velocity over turbine
%Both values are corrected up to the left and right end of the turbine, including cones

%Correct acoustic power dissipation (use .calc instead of .mean)
[results.E2.drop.tubecorr,results.E2.drop.tubecorr_left,results.E2.drop.tubecorr_right] = AcousticPowerCorrection(results,settings);
results.E2.drop.calc = (results.E2.S3S4.calc-results.E2.S1S2.calc)-results.E2.drop.tubecorr';

%Estimated pressure drop over turbine and velocity on either side
[results.p1.drop.calc.complex,results.velocity.ul.complex,results.velocity.ur.complex,results.p1.pl,results.p1.pr] = CalcPressureDrop(results,settings);
results.p1.drop.calc.amp = abs(results.p1.drop.calc.complex);
results.velocity.ul.amp = abs(results.velocity.ul.complex);
results.velocity.ur.amp = abs(results.velocity.ur.complex);
results.velocity.phi_pQ_l = unwrap(phase(results.p1.pl)-phase(results.velocity.ul.complex));
results.velocity.phi_pQ_r = unwrap(phase(results.p1.pr)-phase(results.velocity.ur.complex));

%Calculate impedance
results.imp.l = results.p1.pl./results.velocity.ul.complex;
results.imp.r = results.p1.pr./results.velocity.ur.complex;
results.imp.lr = results.imp.r-results.imp.l;

%% Calculate turbine efficiency and check generator power output versus calibration data (indicate data to be removed if toggled)
[results, settings] = CalcTurbine_performance(results,settings);

%% Show experiment info
if settings.data.batch_processing == 0 %Don't show in batch processing mode
    disp('--------------------------------------------');
    disp(['Title: ' data.Props.Title]);
    disp(['Author: ' data.Props.Author]);
    disp(' ');
    disp(['First logged timestamp: ' datestr(absTime(1))]);
    disp(['Number of timesteps detected: ' num2str(settings.data.Nt)]);
    disp(['Average time interval: ' num2str(settings.data.dtAvg)]);
    disp(['Number of detected total setpoints: ' num2str(settings.data.Nsetpoint)]);
    disp(['Number of detected invalid setpoints: ' num2str(sum(settings.data.setpoint_invalid))]);
    disp(['Generator resistance = ' num2str(settings.data.GeneratorResistance) 'Ohm']);
    disp('--------------------------------------------');
end

%% Plots
%Example of how to make plots with the data, including how to remove invalid data points
%The figures from the publications that use multiple experiments are made with 'Batch_Analysis_multiple_mat_files.m' to easily make them in one go. But they could be made here as well. 
figure(1)
hold all
plot(results.E2.drop.calc(settings.data.setpoint_invalid==0),results.turbine.eff(settings.data.setpoint_invalid==0),'-o')
xlabel('Acoustic power drop')
ylabel('Turbine efficiency')


